<# 
.SYNOPSIS
		Notifies users that their password is about to expire.

.DESCRIPTION
    Let's users know their password will soon expire. Details the steps needed to change their password, and advises on what the password policy requires. Accounts for both standard Default Domain Policy based password policy and the fine grain password policy available in 2008 domains.

.NOTES
    Version    	      : v1.0 - 02/26/2010 - initial version
    									: v2.0 - 08/15/2011 - added email code
    																			- added transcript option
    																			- added demo parameter & formatted output
    																			- added param block with some default values
    																			- moved (Get-AdDomain).DomainMode code to outside of loop to help speed up processing (since it really needs to be called only once)
    																			- added check for ActiveDirectory module & Exchange snapins
    																			- added alert for same day expiration
    																			- auto load or install RSAT-AD-PowerShell feature
    									: v2.1 - 08/31/2011 - added some additional code to the section that installs the RSAT-AD-PowerShell feature
    																			- corrected code that wouldn't send mail until a the user was one day into the reminder window
    																			- cleaned up HTML code indentations to make it a little easier to read
    																			- variable for image path so that editing the HTML is straightforward
    																			- added preview mode to see HTML email
    																			- added install mode to automatically create scheduled task
    																			- updated and enhanced the comment based help
    																			- added code for event log logging
    Wish list					: Better detection of Exchange server
    									: Set $DaysToWarn automatically based on Default Domain GPO setting
    Rights Required		: local admin on server it's running on
    Sched Task Req'd	: Yes - http://www.ucblogs.net/blogs/exchange/archive/2009/10/07/Running-PowerShell-scripts-via-Scheduled-Tasks.aspx
    Lync Version			: N/A
    Exchange Version	: 2007 or later
    Author       			: M. Ali (original AD query), Pat Richard, Exchange MVP
    Email / Blog 			: pat@innervation.com 	http://exchangeblogs.com
    Dedicated Post		: http://www.ucblogs.net/blogs/exchange/archive/2011/08/27/New_2D00_PasswordReminder.ps1-_2D00_-email-users-when-their-password-will-soon-expire.aspx
    Disclaimer   			: You running this script means you won't blame me if this breaks your stuff.
    Info Stolen from 	: (original) http://blogs.msdn.com/b/adpowershell/archive/2010/02/26/find-out-when-your-password-expires.aspx
    									: (date) http://technet.microsoft.com/en-us/library/ff730960.aspx
											: (calculating time) http://blogs.msdn.com/b/powershell/archive/2007/02/24/time-till-we-land.aspx
											: http://social.technet.microsoft.com/Forums/en-US/winserverpowershell/thread/23fc5ffb-7cff-4c09-bf3e-2f94e2061f29/
											: (password decryption) http://social.technet.microsoft.com/Forums/en-US/winserverpowershell/thread/f90bed75-475e-4f5f-94eb-60197efda6c6/

.LINK     
    http://www.ucblogs.net/blogs/exchange/archive/2011/08/27/New_2D00_PasswordReminder.ps1-_2D00_-email-users-when-their-password-will-soon-expire.aspx

.INPUTS
		None. You cannot pipe objects to this script
		
.PARAMETER Demo
		Runs the script in demo mode. No emails are sent to the user(s), and onscreen output includes those who are expiring soon.

.PARAMETER Preview
		Sends a sample email to the user specified. Usefull for testing how the reminder email looks.
		
.PARAMETER PreviewUser
		User name of user to send the preview email message to.

.PARAMETER Install
		Create the scheduled task to run the script daily. It does NOT create the required Exchange receive connector.

.PARAMETER Transcript
		Creates a transcript of the script output.

.PARAMETER Company
		Name of the company to be included in the reminder emails.

.PARAMETER OwaUrl
		Link to Outlook Web Access/App. Included in the reminder emails.

.PARAMETER EmailServer
		IP address of email server that this script sends the messages to for delivery.

.PARAMETER EmailFrom
		Name and address that the reminder emails come from.

.PARAMETER HelpDeskPhone
		Phone number of the Help Desk dept. Included at the bottom of the reminder emails.

.PARAMETER TranscriptFilename
		Name of the file for the transcript. This is automatically set.

.PARAMETER UsersNotified
		Incremental counter of users that reminder emails have been sent to. This is automatically set.

.PARAMETER DaysToWarn
		How far in advance of a password exiring to start emailing the users.

.PARAMETER ImagePath
		Default path to any images included in the HTML of the reminder emails.

.PARAMETER ScriptName
		Automatically set to the name of this script.

.PARAMETER ScriptPathAndName
		Automatically set to the full path and name of this script.

.EXAMPLE 
		.\New-PasswordReminder.ps1
		
		Description
		-----------
		Searches Active Directory for users who have passwords expiring soon, and emails them a reminder with instructions on how to change their password.

.EXAMPLE 
		.\New-PasswordReminder.ps1 -demo
		
		Description
		-----------
		Searches Active Directory for users who have passwords expiring soon, and lists those users on the screen, along with days till expiration and policy setting

.EXAMPLE 
		.\New-PasswordReminder.ps1 -Preview -PreviewUser [username]
		
		Description
		-----------
		Sends the HTML formatted email of the user specified via -PreviewUser. This is used to see what the HTML email will look like to the users.

.EXAMPLE 
		.\New-PasswordReminder.ps1 -install
		
		Description
		-----------
		Creates the scheduled task for the script to run everyday at 6am. It will prompt for the password for the currently logged on user. It does NOT create the required Exchange receive connector.

                               
#> 
#Requires -Version 2.0  
param(
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false)] 
	[switch]$Demo,
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false)] 
	[switch]$Preview,
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false)] 
	[switch]$Install,
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false)] 
	[string]$PreviewUser,
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false)] 
	[switch]$Transcript,
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false, HelpMessage="Please specify a company name.")]
	[ValidateNotNullOrEmpty()] 
	[string]$Company = "Contoso Ltd",
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false, HelpMessage="Please specify an OWA URL")]
	[ValidateNotNullOrEmpty()]
	[string]$OwaUrl = "https://mail.contoso.com",
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false, HelpMessage="Please specify the IP address of your email server")]
	[ValidateNotNullOrEmpty()]
	[string]$EmailServer = "10.9.0.11",
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false, HelpMessage="Please specify a name and email address for the email 'from' field")]
	[ValidateNotNullOrEmpty()]
	[string]$EmailFrom = "Help Desk <helpdesk@contoso.com>",
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false)] 
	[string]$HelpDeskPhone = "(586) 555-1010",	
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false)] 
	[string] $TranscriptFilename = $MyInvocation.MyCommand.Name + " " + (hostname)+ " {0:yyyy-MM-dd hh-mmtt}.log" -f (Get-Date),
	[parameter(ValueFromPipeline=$false, ValueFromPipelineByPropertyName=$false, Mandatory=$false, HelpMessage="This must be zero")]
	[ValidateNotNullOrEmpty()]
	[int]$global:UsersNotified = 0,
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false, HelpMessage="Please specify how many days before expiration that users should begin to be notified.")]
	[ValidateNotNullOrEmpty()]
	[int]$DaysToWarn = 14,
	[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$false)] 
	[string] $ImagePath = "http://www.mydomain.com/scripts",
	[parameter(ValueFromPipeline=$false, ValueFromPipelineByPropertyName=$false, Mandatory=$false)]
	[ValidateNotNullOrEmpty()]
	[string] $ScriptName = $MyInvocation.MyCommand.Name,
	[parameter(ValueFromPipeline=$false, ValueFromPipelineByPropertyName=$false, Mandatory=$false)]
	[ValidateNotNullOrEmpty()]
	[string] $ScriptPathAndName = $MyInvocation.MyCommand.Definition
)

if ($transcript){
	Start-Transcript -path .\$TranscriptFilename | Out-Null
}

function Get-ModuleStatus { 
	param	(
		[parameter(ValueFromPipeline=$true, ValueFromPipelineByPropertyName=$true, Mandatory=$true, HelpMessage="No module name specified!")] 
		[string]$name
	)
	if(!(Get-Module -name "$name")) { 
		if(Get-Module -ListAvailable | ? {$_.name -eq "$name"}) { 
			Import-Module -Name "$name" 
			# module was imported
			return $true | Out-Null
		} else {
			# module was not available (Windows feature isn't installed)
			return $false | Out-Null
		}
	}else {
		# module was already imported
		return $true | Out-Null
	}
} # end function Get-ModuleStatus

function Get-XADUserPasswordExpirationDate {
	Param ([Parameter(Mandatory=$true, Position=0, ValueFromPipeline=$true, HelpMessage="Identity of the Account")]
	[Object] $accountIdentity)
	PROCESS {
		$accountObj = Get-ADUser $accountIdentity -properties PasswordExpired, PasswordNeverExpires, PasswordLastSet, name
		# Make sure the password is not expired, and the account is not set to never expire
    if ((!($accountObj.PasswordExpired)) -and (!($accountObj.PasswordNeverExpires))) {
    	$passwordSetDate = $accountObj.PasswordLastSet
    	# see if the date the password was last set is available
      if ($passwordSetDate -ne $null) {
      	$maxPasswordAgeTimeSpan = $null
        # see if we're at Windows2008 domain functional level, which supports granular password policies
        if ($global:dfl -ge 3) { # 2008 Domain functional level
          $accountFGPP = Get-ADUserResultantPasswordPolicy $accountObj
          if ($accountFGPP -ne $null) {
          	$maxPasswordAgeTimeSpan = $accountFGPP.MaxPasswordAge
					} else {
						$maxPasswordAgeTimeSpan = (Get-ADDefaultDomainPasswordPolicy).MaxPasswordAge
					}
				} else { # 2003 or ealier Domain Functional Level
					$maxPasswordAgeTimeSpan = (Get-ADDefaultDomainPasswordPolicy).MaxPasswordAge
				}
				if ($maxPasswordAgeTimeSpan -eq $null -or $maxPasswordAgeTimeSpan.TotalMilliseconds -ne 0) {
					$DaysTillExpire = [math]::round(((New-TimeSpan -Start (Get-Date) -End ($passwordSetDate + $maxPasswordAgeTimeSpan)).TotalDays),0)
					if ($preview){$DaysTillExpire = 1}
					if ($DaysTillExpire -le $DaysToWarn){
						$PolicyDays = [math]::round((($maxPasswordAgeTimeSpan).TotalDays),0)
						if ($demo)	{write-host ("{0,-25}{1,-8}{2,-12}" -f $accountObj.Name, $DaysTillExpire, $PolicyDays)}
            # start assembling email to user here
						$EmailName = $accountObj.Name						
						$DateofExpiration = (Get-Date).AddDays($DaysTillExpire)
						$DateofExpiration = (Get-Date($DateofExpiration) -f d)
						
[string]$emailbody = @"
<html>
	<head>
		<meta http-equiv="Content-Type" content="text/html; charset=utf-8">
	</head>
<body>
	<table id="email" border="0" cellspacing="0" cellpadding="0" width="655" align="center">
		<tr>
			<td colspan="4" align="left" valign="top"><img src="$ImagePath/spacer.gif" alt="" width="46" height="28" align="absMiddle"><font style="font-size: 10px; color: #000000; line-height: 16px; font-family: Verdana, Arial, Helvetica, sans-serif">If this e-mail does not appear properly, please <a href="$HelpDeskURL" style="font-weight: bold; font-size: 10px; color: #cc0000; font-family: verdana, arial, helvetica, sans-serif; text-decoration: underline" target="_blank">click here</a>.</font></td>
		</tr>
		<tr>
			<td colspan="4" height="121" align="left" valign="bottom"><a href="$HelpDeskURL" target="_blank"><img src="$ImagePath/header.gif" border="0" alt="Header" width="655" height="121"></a></td>
		</tr>
		<tr>
			<td colspan="4">
				<table id="body" border="0" cellspacing="0" cellpadding="0">
					<tr>
						<td width="1" align="left" valign="top" bgcolor="#a8a9ad"><img src="$ImagePath/spacer50.gif" alt="" width="1" height="50"></td>
						<td><img src="$ImagePath/spacer.gif" alt="" width="46" height="106"></td>
						<td id="text" width="572" align="left" valign="top" style="font-size: 12px; color: #000000; line-height: 17px; font-family: Verdana, Arial, Helvetica, sans-serif">
"@

if ($DaysTillExpire -le 1){
	$emailbody += @"
		<div align='center'>
			<table border='0' cellspacing='0' cellpadding='0' style='width:510px; background-color: white; border: 0px;'>
				<tr>
					<td align='right'><img width='36' height='28' src='$ImagePath/image001b.gif' alt='Description: $ImagePath/image001b.gif'></td>		
					<td style="font-family: verdana; background: #E12C10; text-align: center; padding: 0px; font-size: 9.0pt; color: white">ALERT: You must change your password today or you will be locked out!</td>		
					<td align='left'><img border='0' width='14' height='28' src='$ImagePath/image005b.gif' alt='Description: $ImagePath/image005b.gif'></td>
				</tr>
			</table>
		</div>
"@
}

$emailbody += @"
			<p style="font-weight: bold">Hello, $EmailName,</p>
			<p>It's change time again! Your $company password expires in <span style="background-color: red; color: white; font-weight: bold;">&nbsp;$DaysTillExpire&nbsp;</span> day(s), on $DateofExpiration.</p>
			<p>Please use one of the methods below to update your password:</p>
			<ol>
				<li>$company office computers and Terminal Server users: You may update your password on your computer by pressing Ctrl-Alt-Delete and selecting 'Change Password' from the available options. If you use a $company laptop in addition to a desktop PC, be sure and read #3 below.</li>
				<li>Remote Outlook Client, Mac, and/or Outlook Web App users: If you only access our email system, please use the following method to easily change your password:</li>
				<ul>
					<li>Log into <a href="$owaurl">Outlook Web App</a> using Internet Explorer (PC) or Safari or Firefox (Mac).</li>
					<li>Click on the Options button in the upper right corner of the page.</li>		
					<li>Select the &quot;Change Password&quot; link to change your password.</li>
					<li>Enter your current password, then your new password twice, and click Save</li>
					<li><span style="font-weight: bold">NOTE:</span> You will now need to use your new password when logging into Outlook Web App, Outlook 2010, SharePoint, Windows Mobile (ActiveSync) devices, etc. Blackberry Enterprise Users (BES) will not need to update their password. Blackberry Internet Service (BIS) users will be required to use their new password on their device.</li>
				</ul>
				<li>$company issued laptops: If you have been issued a $company laptop, you must be in a corporate office and directly connected to the company network to change your password. If you also use a desktop PC in the office, you must remember to always update your domain password on the laptop first. Your desktop will automatically use the new password.</li>
				<ul>
					<li>Log in on laptop</li>
					<li>Press Ctrl-Alt-Delete and select 'Change Password' from the available options.</li>
					<li>Make sure your workstation (if you have one) has been logged off any previous sessions so as to not cause conflict with your new password.</li>
				</ul>
			</ol>
			<p>Think you've got a complex password? Run it through the <a href="http://www.passwordmeter.com/">The Password Meter</a></p>
			<p>Think your password couldn't easily be hacked? See how long it would take: <a href="http://howsecureismypassword.net/">How Secure Is My Password</a></p>
			<p>Remember, if you do not change your password before it expires on $DateofExpiration, you will be locked out of all $company Computer Systems until an Administrator unlocks your account.</p>
			<p>If you are traveling or will not be able to bring your laptop into the office before your password expires, please call the number below for additional instructions.</p>
			<p>You will continue to receive these emails daily until the password is changed or expires.</p>

			<p>Thank you,<br />
			The $company Help Desk<br />
			$HelpDeskPhone</p>
 
			<table style="background-color: #dedede; border: 1px solid black">
				<tr>
					<td style="font-size: 12px; color: #000000; line-height: 17px; font-family: Verdana, Arial, Helvetica, sans-serif"><b>$company Password Policy</b>
						<ul>
							<li>Your password must have a minimum of a 8 characters.</li>
							<li>You may not use a previous password.</li>
							<li>Your password must not contain parts of your first, last, or logon name.</li>
							<li>Your password must be changed every $PolicyDays days.</li>
							<li>Your password requires a minimum of three of the following four categories:</li>
							<ul>
								<li>1 upper case character (A-Z)</li>
								<li>1 lower case character (a-z)</li>
								<li>1 numeric character (0-9)</li>
								<li>1 special character ( ~!@#%^, etc.)</li>
							</ul>
						</ul>
					</td>
				</tr>
			</table>
								
							</td>
							<td width="49" align="left" valign="top"><img src="$ImagePath/spacer50.gif" alt="" width="49" height="50"></td>
							<td width="1" align="left" valign="top" bgcolor="#a8a9ad"><img src="$ImagePath/spacer50.gif" alt="" width="1" height="50"></td>
						</tr>
					</table>
					<table id="footer" border="0" cellspacing="0" cellpadding="0" width="655">
						<tr>
							<td colspan="3"><img src="$ImagePath/footer.gif" alt="© 2010 $company" width="655" height="81"></td>
						</tr>
					</table>
					<table border="0" cellspacing="0" cellpadding="0" width="655" align="center">
						<tr>
							<td align="left" valign="top"><img src="$ImagePath/spacer.gif" alt="" width="36" height="1"></td>
							<td align="left" valign="top"><font face="Verdana" size="1" color="#000000"><p>This email was sent by an automated process. If you would like to comment on it, please visit <a href="$HelpDeskURL"><font color="#ff0000"><u>click here</u></font></a>.</p>
								<p style="color: #009900;"><font face="Webdings" size="4">P</font> Please consider the environment before printing this email.</p></font>
							</td>
							<td align="left" valign="top"><img src="$ImagePath/spacer.gif" alt="" width="36" height="1"></td>
						</tr>
					</table>
				</td>
			</tr>
		</table>
	</body>
</html>
"@
						if (!($demo)){
							Send-MailMessage -To (Get-User $accountObj.Name).WindowsEmailAddress -Subject "Your password expires in $DaysTillExpire day(s)" -Body $emailbody -From $EmailFrom -SmtpServer $EmailServer -Priority High -BodyAsHtml
							$global:UsersNotified++
						}
					}
				}
			}
		}
	}
} # end function Get-XADUserPasswordExpirationDate

# let's clean up some variables at the end so we free up resources
function Remove-ScriptVariables($path) {  
	$result = Get-Content $path |  
	ForEach { if ( $_ -match '(\$.*?)\s*=') {      
			$matches[1]  | ? { $_ -notlike '*.*' -and $_ -notmatch 'result' -and $_ -notmatch 'env:'}  
		}  
	}  
	ForEach ($v in ($result | Sort-Object | Get-Unique)){		
		Remove-Variable ($v.replace("$","")) -ErrorAction SilentlyContinue
	}
} # end function Get-ScriptVariables

function Install	{
	# http://technet.microsoft.com/en-us/library/cc725744(WS.10).aspx
	$error.clear()
	Write-Host "Creating scheduled task `"$ScriptName`"..."
	$TaskPassword = Read-Host "Please enter the password for $env:userdomain\$env:username" -AsSecureString
	$TaskPassword = [System.Runtime.InteropServices.Marshal]::PtrToStringAuto([System.Runtime.InteropServices.Marshal]::SecureStringToBSTR($TaskPassword))
	schtasks /create /tn $ScriptName /tr "$env:windir\system32\windowspowershell\v1.0\powershell.exe -psconsolefile $env:ExchangeInstallPath\Bin\exshell.psc1 -command $ScriptPathAndName" /sc Daily /st 06:00 /ru $env:userdomain\$env:username /rp $TaskPassword | Out-Null
	if (!($error)){
		Write-Host "done!" -ForegroundColor green
	}else{
		Write-Host "failed!" -ForegroundColor red
	}
	exit
} # end function Install

if ($install){
	Install
}

if ((Get-ModuleStatus ActiveDirectory) -eq $false){
	$error.clear()
	Write-Host "Installing the Active Directory module..." -ForegroundColor yellow
	Get-ModuleStatus ServerManager
	Add-WindowsFeature RSAT-AD-PowerShell
	if ($error){
		Write-Host "Active Directory module could not be installed. Exiting..." -ForegroundColor red; 
		if ($transcript){Stop-Transcript}
		exit
	}
}
$global:dfl = (Get-AdDomain).DomainMode
# Get-ADUser -filter * -properties PasswordLastSet,EmailAddress,GivenName -SearchBase "OU=Users,DC=domain,DC=test" |foreach {
if (!($preview)){
	$users = Get-AdUser -filter * -ResultSetSize $null
}else{
	$users = Get-AdUser $PreviewUser
}
if ($demo){
	# $WhatIfPreference = $true
	Write-Host "`n"
	Write-Host ("{0,-25}{1,-8}{2,-12}" -f "User", "Expires", "Policy") -ForegroundColor cyan
	Write-Host ("{0,-25}{1,-8}{2,-12}" -f "========================", "=======", "===========") -ForegroundColor cyan
}

$evt=new-object System.Diagnostics.EventLog("Application")
$evt.Source=$ScriptName
$infoevent=[System.Diagnostics.EventLogEntryType]::Information
$EventLogText = "Beginning processing"
$evt.WriteEntry($EventLogText,$infoevent,70)

ForEach ($user in $users){
	Get-XADUserPasswordExpirationDate $user.samaccountname
}

$EventLogText = "Finished processing $global:UsersNotified account(s). `n`nFor more information about this script, run Get-Help .\$ScriptName. See the website at http://www.ucblogs.net/blogs/exchange/archive/2011/08/27/New_2D00_PasswordReminder.ps1-_2D00_-email-users-when-their-password-will-soon-expire.aspx"
$evt.WriteEntry($EventLogText,$infoevent,70)

# $WhatIfPreference = $false
if ($transcript){
	Stop-Transcript | Out-Null
}
Remove-ScriptVariables($MyInvocation.MyCommand.Name)